@@ -2,6 +2,7 @@
 
 from __future__ import annotations
 
+import hashlib
 import json
 import os
 import subprocess
@@ -12,12 +13,13 @@ import typer
 from typing_extensions import Annotated, Optional
 
 import calkit
-from calkit.cli import print_sep, run_cmd
+from calkit.cli import print_sep, raise_error, run_cmd
 from calkit.cli.config import config_app
 from calkit.cli.import_ import import_app
 from calkit.cli.list import list_app
 from calkit.cli.new import new_app
 from calkit.cli.notebooks import notebooks_app
+from calkit.cli.office import office_app
 
 app = typer.Typer(
     invoke_without_command=True,
@@ -26,12 +28,16 @@ app = typer.Typer(
     pretty_exceptions_show_locals=False,
 )
 app.add_typer(config_app, name="config", help="Configure Calkit.")
+app.add_typer(new_app, name="new", help="Create a new Calkit object.")
 app.add_typer(
-    new_app, name="new", help="Add new Calkit object (to calkit.yaml)."
+    new_app,
+    name="create",
+    help="Create a new Calkit object (alias for 'new').",
 )
 app.add_typer(notebooks_app, name="nb", help="Work with Jupyter notebooks.")
 app.add_typer(list_app, name="list", help="List Calkit objects.")
 app.add_typer(import_app, name="import", help="Import objects.")
+app.add_typer(office_app, name="office", help="Work with Microsoft Office.")
 
 
 @app.callback()
@@ -46,6 +52,53 @@ def main(
         raise typer.Exit()
 
 
+@app.command(name="clone")
+def clone(
+    url: Annotated[str, typer.Argument(help="Repo URL.")],
+    location: Annotated[
+        str,
+        typer.Argument(
+            help="Location to clone to (default will be ./{repo_name})"
+        ),
+    ] = None,
+    no_config_remote: Annotated[
+        bool,
+        typer.Option(
+            "--no-config-remote",
+            help="Do not automatically configure Calkit DVC remote.",
+        ),
+    ] = False,
+    no_dvc_pull: Annotated[
+        bool, typer.Option("--no-dvc-pull", help="Do not pull DVC objects.")
+    ] = False,
+):
+    """Clone a Git repo and by default configure and pull from the DVC
+    remote.
+    """
+    # Git clone
+    cmd = ["git", "clone", url]
+    if location is not None:
+        cmd.append(location)
+    try:
+        subprocess.call(cmd)
+    except Exception as e:
+        raise_error(str(e))
+    if location is None:
+        location = url.split("/")[-1].removesuffix(".git")
+    typer.echo(f"Moving into repo dir: {location}")
+    os.chdir(location)
+    # Setup auth for any Calkit remotes
+    if not no_config_remote:
+        remotes = calkit.dvc.get_remotes()
+        for name, url in remotes.items():
+            if name == "calkit" or name.startswith("calkit:"):
+                typer.echo(f"Setting up authentication for DVC remote: {name}")
+                calkit.dvc.set_remote_auth(remote_name=name)
+    # DVC pull
+    if not no_dvc_pull:
+        subprocess.call(["dvc", "pull"])
+
+
 @app.command(name="status")
 def get_status():
     """Get a unified Git and DVC status."""
@@ -97,8 +150,7 @@ def add(
     adding any .dvc files to Git when adding to DVC.
     """
     if to is not None and to not in ["git", "dvc"]:
-        typer.echo(f"Invalid option for 'to': {to}")
-        raise typer.Exit(1)
+        raise_error(f"Invalid option for 'to': {to}")
     # Ensure autostage is enabled for DVC
     subprocess.call(["dvc", "config", "core.autostage", "true"])
     subprocess.call(["git", "add", ".dvc/config"])
@@ -107,6 +159,9 @@ def add(
     else:
         dvc_extensions = [
             ".png",
+            ".jpeg",
+            ".jpg",
+            ".gif",
             ".h5",
             ".parquet",
             ".pickle",
@@ -114,16 +169,18 @@ def add(
             ".avi",
             ".webm",
             ".pdf",
+            ".xlsx",
+            ".docx",
+            ".xls",
+            ".doc",
         ]
         dvc_size_thresh_bytes = 1_000_000
         if "." in paths and to is None:
-            typer.echo("Cannot add '.' with calkit; use git or dvc")
-            raise typer.Exit(1)
+            raise_error("Cannot add '.' with calkit; use git or dvc")
         if to is None:
             for path in paths:
                 if os.path.isdir(path):
-                    typer.echo("Cannot auto-add directories; use git or dvc")
-                    raise typer.Exit(1)
+                    raise_error("Cannot auto-add directories; use git or dvc")
         repo = git.Repo()
         for path in paths:
             # Detect if this file should be tracked with Git or DVC
@@ -250,7 +307,7 @@ def run_server():
         port=8866,
         host="localhost",
         reload=True,
-        reload_dirs=[os.path.dirname(__file__)],
+        reload_dirs=[os.path.dirname(os.path.dirname(__file__))],
     )
 
 
@@ -320,11 +377,10 @@ def run_dvc_repro(
         args += ["--pipeline", pipeline]
     if downstream is not None:
         args += downstream
-    subprocess.call(["dvc", "repro"] + args)
+    subprocess.check_call(["dvc", "repro"] + args)
     # Now parse stage metadata for calkit objects
     if not os.path.isfile("dvc.yaml"):
-        typer.echo("No dvc.yaml file found")
-        raise typer.Exit(1)
+        raise_error("No dvc.yaml file found")
     objects = []
     with open("dvc.yaml") as f:
         pipeline = calkit.ryaml.load(f)
@@ -332,21 +388,18 @@ def run_dvc_repro(
             ckmeta = stage_info.get("meta", {}).get("calkit")
             if ckmeta is not None:
                 if not isinstance(ckmeta, dict):
-                    typer.echo(
+                    raise_error(
                         f"Calkit metadata for {stage_name} is not a dictionary"
                     )
-                    typer.Exit(1)
                 # Stage must have a single output
                 outs = stage_info.get("outs", [])
                 if len(outs) != 1:
-                    typer.echo(
+                    raise_error(
                         f"Stage {stage_name} does not have exactly one output"
                     )
-                    raise typer.Exit(1)
                 cktype = ckmeta.get("type")
                 if cktype not in ["figure", "dataset", "publication"]:
-                    typer.echo(f"Invalid Calkit output type '{cktype}'")
-                    raise typer.Exit(1)
+                    raise_error(f"Invalid Calkit output type '{cktype}'")
                 objects.append(
                     dict(path=outs[0]) | ckmeta | dict(stage=stage_name)
                 )
@@ -433,21 +486,24 @@ def run_in_env(
     ck_info = calkit.load_calkit_info()
     envs = ck_info.get("environments", {})
     if not envs:
-        typer.echo("No environments defined in calkit.yaml", err=True)
-        raise typer.Exit(1)
+        raise_error("No environments defined in calkit.yaml")
     if isinstance(envs, list):
-        typer.echo(
-            "Error: Environments should be a dict, not a list", err=True
-        )
-        raise typer.Exit(1)
-    if len(envs) > 1 and env_name is None:
-        typer.echo(
-            "Environment must be specified if there are multiple",
-            err=True,
-        )
-        raise typer.Exit(1)
+        raise_error("Error: Environments should be a dict, not a list")
     if env_name is None:
-        env_name = list(envs.keys())[0]
+        # See if there's a default env, or only one env defined
+        default_env_name = None
+        for n, e in envs.items():
+            if e.get("default"):
+                if default_env_name is not None:
+                    raise_error(
+                        "Only one default environment can be specified"
+                    )
+                default_env_name = n
+        if default_env_name is None and len(envs) == 1:
+            default_env_name = list(envs.keys())[0]
+        env_name = default_env_name
+    if env_name is None:
+        raise_error("Environment must be specified if there are multiple")
     env = envs[env_name]
     cwd = os.getcwd()
     image_name = env.get("image", env_name)
@@ -477,8 +533,7 @@ def run_in_env(
             typer.echo(f"Running command: {cmd}")
         subprocess.call(cmd)
     else:
-        typer.echo("Environment kind not supported", err=True)
-        raise typer.Exit(1)
+        raise_error("Environment kind not supported")
 
 
 @app.command(
@@ -507,8 +562,7 @@ def check_call(
             subprocess.check_call(if_error, shell=True)
             typer.echo("Fallback call succeeded")
         except subprocess.CalledProcessError:
-            typer.echo("Fallback call failed", err=True)
-            raise typer.Exit(1)
+            raise_error("Fallback call failed")
 
 
 @app.command(
@@ -530,6 +584,7 @@ def build_docker(
         _ = out[0].pop("Id")
         _ = out[0].pop("RepoDigests")
         _ = out[0].pop("Metadata")
+        _ = out[0].pop("DockerVersion")
         return out
 
     typer.echo(f"Checking for existing image with tag {tag}")
@@ -539,6 +594,10 @@ def build_docker(
     except subprocess.CalledProcessError:
         typer.echo(f"No image with tag {tag} found locally")
         inspect = []
+    typer.echo(f"Reading Dockerfile from {fpath}")
+    with open(fpath) as f:
+        dockerfile = f.read()
+    dockerfile_md5 = hashlib.md5(dockerfile.encode()).hexdigest()
     lock_fpath = fpath + "-lock.json"
     rebuild = True
     if os.path.isfile(lock_fpath):
@@ -549,11 +608,14 @@ def build_docker(
         typer.echo(f"Lock file ({lock_fpath}) does not exist")
         lock = None
     if inspect and lock:
-        typer.echo("Checking image against lock file")
-        rebuild = inspect[0]["RootFS"]["Layers"] != lock[0]["RootFS"]["Layers"]
+        typer.echo("Checking image and Dockerfile against lock file")
+        rebuild = inspect[0]["RootFS"]["Layers"] != lock[0]["RootFS"][
+            "Layers"
+        ] or dockerfile_md5 != lock[0].get("DockerfileMD5")
     if rebuild:
         subprocess.check_call(["docker", "build", "-t", tag, "-f", fpath, "."])
     # Write the lock file
     inspect = get_docker_inspect()
+    inspect[0]["DockerfileMD5"] = dockerfile_md5
     with open(lock_fpath, "w") as f:
         json.dump(inspect, f, indent=4)
